%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for estimating nxn matrix B_0 such    %
%          that Omega = B_0*B_0' and all other parameters (if    %
%          required) simultaneously. Provides a FIML estimator   %
%          of all requested parameters                           %
%                                                                %
%          Function: fiml                                        %
%                                                                %
%          Input:                                                %
%                       tm: structure with model info            %
%                       type_var: string determining type of     %
%                           restrictions to use                  %
%                       keep_rest: restrictions that can be      %
%                           put on B_0 alone                     %
%                       del_rest: restrictions that cannot be    %
%                           put on B_0 alone                     %
%                       use_psi_rest: restrictions on C*B_0      %
%                           which are linearly independent of    %
%                           restrictions placed on [Psi alpha]   %
%                           when they are linked to [Psi alpha]  %
%                       Psi: parameters on short-run dynamics    %
%                           and deterministics                   %
%                       alpha: parameters on levels relations    %
%                       beta: cointegration space                %
%                       epsilon: residuals                       %
%                       Z0: endogenous variables in changes      %
%                       Z1: levels variables                     %
%                       Z2: changes and deterministics           %
%                       B_init: initial values for B_0           %
%                       waitbar: 0-1 indicator whether to        %
%                           display a progress dialog or not     %
%                                                                %
%          Output:                                               %
%                        status: indicator of success            %
%                           (0 OK, 1 failure, 2 bad initial      %
%                            B matrix)                           %
%                        abort: string indicating if estimation  %
%                           was aborted or not                   %
%          		           B_0: nxn matrix with structural         %
%                           coefficients                         %
%                        Psi_n: nxk matrix with parameters on    %
%                           deterministic variables, lagged      %
%                           first differences, and exogenous     %
%                           I(0) variables                       %
%                        alpha_n: nxr matrix with weights on the %
%                           cointegration relations              %
%                        beta_n: (n+q1+dcoint)xr matrix with     %
%                           cointegration space parameters       %
%                        C_n: nxn matrix with reduced form       %
%                           parameters on the trends             %
%                        A: nxn matrix with structural long-run  %
%                           coefficients on the trends           %
%                        stdbeta: estimated asymptotic           %
%                           conditional standard errors for beta %
%                        S: n^2xq matrix mapping free parameters %
%                           gamma into vec(B_0)                  %
%                        Sigma_theta: covariance matrix for      %
%                           Psi and alpha                        %
%                        Sigma_B0: covariance matrix for B_0     %
%                        Sigma_B0_adj: adjusted covariance       %
%                        matrix for B_0                          %
%                        Sigma_A: covariance matrix for A        %
%                        Sigma_full: Complete covariance matrix  %
%                           of all the model parameters (except  %
%                           for beta)                            %
%                        infomat: 1x2 vector with number of      %
%                           iterations and value of convergence  %
%                           criterion                            %
%                        Rmat: the restrictions matrix           %
%                        epsilon_new: nxT matrix with reduced    %
%                           form residuals                       %
%                        num_rest: the actual number of          %
%                           restrictions implied by the          %
%                           identifying assumptions              %
%                        dC_theta: the derivative of C with      %
%                           respect to vec([Psi alpha])          %
%                                                                %
%          This version:        June 16, 2006                    %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [status,abort,B_0,Psi_n,alpha_n,beta_n,C_n,A,stdbeta,S,Sigma_theta,Sigma_B0,Sigma_B0_adj,Sigma_A,infomat,Rmat,epsilon_new,num_rest,dC_theta,dB_theta] = fiml(tm,type_var,keep_rest,del_rest,use_psi_rest,Psi,alpha,beta,epsilon,Z0,Z1,Z2,B_init,waitbar);

if nargin<13;
   waitbar = 0;
end;

r = size(alpha,2);
[n T] = size(epsilon);
Omega = (1/T)*epsilon*epsilon';

local_status = 0;
status = 1;
abort = '';
B_0 = [];
A = [];
S = [];
Sigma_B0 = [];
Sigma_B0_adj = [];
Sigma_A = [];
Sigma_full = [];
Sigma_theta = [];
infomat = [];
R_l = [];
Rmat = [];
stdbeta = [];
epsilon_new = epsilon;
num_rest = [];
dC_theta = [];
dB_theta = [];
Psi_n = [];
alpha_n = [];
beta_n = [];
C_n = [];
num_rest = [];

%
% check the ini file
%

curr_ini = find_inisettings;

if curr_ini.convcrit<10;
   ch_convcrit = ['1.0e-00' num2str(curr_ini.convcrit)];
else;
   ch_convcrit = ['1.0e-0' num2str(curr_ini.convcrit)];
end;
ccrit = mystr2num(ch_convcrit);

%
% initialize B if it is not already given
%

if isempty(B_init)==1;
   [U,V] = eig(Omega);
   B_init = U*sqrt(V);
else;
   %
   % check to make sure B_init is non-singular
   %
   if rank(B_init)~=n;
      [U,V] = eig(Omega);
      B_init = U*sqrt(V);
      local_status = 2;
   end;
end;

%
% retrieving restrictions placed on alpha
%

if tm.restrict(1,1)==1;
   file = strcat(tm.modelsdir,'G-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   G = loaddata(file);
   [Rmat_alpha,rvec_alpha] = rmatrix(G,n,r);
   if isempty(tm.Z2)==0;
      Rmat_alpha = [zeros(size(G,1),size(tm.Z2,1)*n) Rmat_alpha];
   end;
else;
   Rmat_alpha = [];
   rvec_alpha = [];
end;

%
% we need to rescale alpha and beta to make sure the cointegration relations
% are roughly on the same scale as the first difference variables. ones the
% algorithm has finished we need to rescale alpha and the estimated asymptotic
% covariance matrix
%

if (r>0)&(tm.restrict(1,2)==0);
   beta_fix = beta(1:r,:);
   beta = beta/beta_fix;
   alpha = alpha*beta_fix';
end;

%
% retrieve restrictions on beta (only possible when r>0 and r<n+q1+dcoint
%

if (curr_ini.reestbeta==1)&(tm.restrict(1,2)==1);
   sivec = zeros(r,1);
   %
   % bH is needed to check the gradient of the likelihood function
   % wrt the beta parameters
   %
   for i=1:r;
      file = strcat(tm.modelsdir,'H_',num2str(i),'-',num2str(n),'x',num2str(r),'x',num2str(tm.dcoint+tm.q1),'-',tm.modelfile,'.rest');
      fid = fopen(file,'rt');
      if fid>-1;
         fclose(fid);
         h = loaddata(file);
         if i==1;
            H = h;
            bH = h;
         else;
            H = [H h];
            bH = [[bH zeros(size(bH,1),size(h,2))];[zeros(size(h,1),size(bH,2)) h]];
         end;
         sivec(i,1) = size(h,2);
      end;
   end;
   %
   % Get the normalizing variables
   %
   file = strcat(tm.modelsdir,'normalize-',num2str(n),'x',num2str(r),'x',num2str(tm.dcoint+tm.q1),'-',tm.modelfile,'.norm');
   fid = fopen(file,'rt');
   if fid>-1;
      fclose(fid);
      normvec = loaddata(file);
   else;
      normvec = [];
   end;
   %
   % Setting up the normalizing restrictions
   %
   if isempty(normvec)==0;
      nq1 = size(beta,1);
      addrestmat = zeros(nq1*r,r);
      for i=1:r;
         thisnorm = normvec(i,1);
         addrestmat(thisnorm+nq1*(i-1),i) = 1;
      end;
   else;
      addrestmat = [];
   end;
   %
   % constructing restriction matrices for beta
   %
   Knr = commutation(n+tm.q1+tm.dcoint,r);
   bH_ort = null(bH','r');
   bH_ort = [bH_ort addrestmat];
   bRmat = bH_ort'*Knr';
   if isempty(addrestmat)==1;
      brvec = zeros(size(bRmat,1),1);
   else;
      brvec = [zeros(size(bRmat,1)-r,1);ones(r,1)];
   end;
   bH = null(bH_ort','r');
   %
   % computing some matrices we'll need later on but don't need
   % to recompute
   %
   M_01 = (1/T)*Z0*Z1';
   M_11 = (1/T)*Z1*Z1';
   if isempty(Z2)==0;
      M_21 = (1/T)*Z2*Z1';
      M_12 = M_21';
      M_22 = (1/T)*Z2*Z2';
   else;
      M_21 = [];
      M_12 = [];
      M_22 = [];
   end;
end;

%
% initializing restrictions and certain parameters
%

if strcmp(type_var,'short_run')==1;
   file = strcat(tm.modelsdir,'F-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   F = loaddata(file);
   [Rmat_sr,rvec_sr] = rmatrix(F,n,n);
   R_l = [];
   Rmat_lr = [];
   rvec_lr = [];
   if isempty(Rmat_alpha)==1;
      if isempty([Psi alpha])==0;
         Rmat = [zeros(size(Rmat_sr,1),size([Psi alpha],2)*n) Rmat_sr];
      else;
         Rmat = Rmat_sr;
      end;
      rvec = rvec_sr;
      Rmat_psi = [];
      rvec_psi = [];
   else;
      Rmat = [[Rmat_alpha zeros(size(Rmat_alpha,1),n*n)];[zeros(size(Rmat_sr,1),size(Rmat_alpha,2)) Rmat_sr]];
      rvec = [rvec_alpha;rvec_sr];
      Rmat_psi = Rmat_alpha;
      rvec_psi = rvec_alpha;
   end;
   Rmat_B = Rmat_sr;
   rvec_B = rvec_sr;
   if r<n;
      C = cmat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0);
   else;
      C = cmat_stat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0,[]);
   end;
   xi = xi_mat(C,Psi,alpha,tm.p,tm.pe,r,tm.d,tm.q1,tm.q0);
elseif strcmp(type_var,'long_run')==1;
   file = strcat(tm.modelsdir,'LF-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   LF = loaddata(file);
   [Rmat_lr,rvec_lr] = rmatrix(LF,n,n);
   Rmat_sr = [];
   rvec_sr = [];
   %
   % need to compute a C matrix also for this case, i.e., r==n
   %
   if r==0;
      C = cmat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0,[]);
   else;
      %
      % now r=n
      %
      C = cmat_stat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0,[]);
   end;
   xi = xi_mat(C,Psi,alpha,tm.p,tm.pe,r,tm.d,tm.q1,tm.q0);
else;
   file = strcat(tm.modelsdir,'F-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   F = loaddata(file);
   [Rmat_sr,rvec_sr] = rmatrix(F,n,n);
   file = strcat(tm.modelsdir,'LF-',int2str(n),'x',int2str(r),'-',tm.modelfile,'.rest');
   LF = loaddata(file);
   [Rmat_lr,rvec_lr] = rmatrix(LF,n,n);
   if r<n;
      C = cmat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0);
   else;
      C = cmat_stat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0,[]);
   end;
   xi = xi_mat(C,Psi,alpha,tm.p,tm.pe,r,tm.d,tm.q1,tm.q0);
   %
end;

Nn = ncommutation(n);

%
% making sure B_0 satisfies as many restrictions as possible
%

if strcmp(type_var,'short_run')==0;
   Rmat_B = [Rmat_sr;Rmat_lr*kron(eye(n),C)];
   [red_rest,use_rest] = getdelvars(Rmat_B',[]);
   Rmat_B = Rmat_B(use_rest,:);
   rvec_B = [rvec_sr;rvec_lr];
   rvec_B = rvec_B(use_rest,1);
else;
   Rmat_B = Rmat_sr;
   rvec_B = rvec_sr;
end;

S_B = null(Rmat_B);
d_B = (Rmat_B'/(Rmat_B*Rmat_B'))*rvec_B;
gamma = (S_B'*S_B)\(S_B'*(vec(B_init)-d_B));
B_0 = vecinv(S_B*gamma+d_B,n);
B_0 = B_init;

%
% scale B and epsilon such that they have approximately the same magnitude
% as [Psi alpha]
%


%
% initializing
%

waitbartext = 'ML Estimation of B';
conv_crit = 1;
it = 0;
options = optimset('LargeScale','on','TolCon',ccrit,'TolX',ccrit,'TolFun',ccrit,'MaxIter',curr_ini.maxit,'MaxFunEvals',100*curr_ini.maxit,'GradObj','on','GradConstr','on');
phi = vec([Psi alpha B_0]);

%
% set up a waitbar if asked for
%

if waitbar==1;
   inside_waitbar = 1;
else;
   inside_waitbar = 0;
end;

if (waitbar==1)&(curr_ini.reestbeta==1)&(tm.restrict(1,2)==1);
   input_var.title = ['Progress for maximum of ' int2str(curr_ini.maxit) ' iterations'];
   input_var.facecolor = curr_ini.progress_facecolor;
   input_var.startfacecolor = curr_ini.progress_startfacecolor;
   input_var.edgecolor = curr_ini.progress_edgecolor;
   input_var.bgcolor = curr_ini.progress_bgcolor;
   input_var.stop = 1;
   input_var.clock = 0;
   input_var.name = 'ML Estimation of B';
   mywait = progressdlg(0,input_var);
%   set(mywait,'Name','ML Estimation of B');
   set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
   drawnow;
   inside_waitbar = 0;
end;

while (it<=curr_ini.maxit)&(conv_crit>ccrit);
   it = it+1;
   if (inside_waitbar==0)&(waitbar==1);
      abort = get(mywait,'UserData');
      if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
         break;
      else;
         progressdlg([it/curr_ini.maxit conv_crit]);
      end;
   end;
   %


%   [f,g] = fiml_max_fcn(phi,tm,type_var,Z0,Z1,Z2,beta,r,Nn,Rmat_alpha,rvec_alpha,Rmat_lr,rvec_lr,Rmat_sr,rvec_sr);

%   [c,ceq,GC,GCeq] = fiml_con_fcn(phi,tm,type_var,Z0,Z1,Z2,beta,r,Nn,Rmat_alpha,rvec_alpha,Rmat_lr,rvec_lr,Rmat_sr,rvec_sr);

   [phi,lnl_val,exitflag,output,lambda,grad] = fmincon_fcn('fiml_max_fcn',phi,[],[],[],[],[],[],'fiml_con_fcn',options,inside_waitbar,waitbartext,tm,type_var,Z0,Z1,Z2,beta,r,Nn,Rmat_alpha,rvec_alpha,Rmat_lr,rvec_lr,Rmat_sr,rvec_sr);

   %[x,fval,exitflag,output,lambda,grad,hessian] = fmincon_fcn('myfun',[-1 1],[],[],[],[],[],[],'mycon',options,inside_waitbar,waitbartext,p1,p2);
   conv_crit = output.convergence;
   my_abort = output.abort;

   conv_crit = 0;


end;


Theta = vecinv(phi,n);

if isempty(Z2)==0;
   Psi = Theta(:,1:size(Z2,1));
   if r>0;
      alpha = Theta(:,size(Theta,2)-(n+r)+1:size(Theta,2)-n);
   else;
      alpha = [];
   end;
else;
   alpha = Theta(:,1:r);
   Psi = [];
end;
B = Theta(:,size(Theta,2)-n+1:size(Theta,2));

if (isempty(Z2)==0)&(r>0);
  epsilon = Z0-(Psi*Z2)-(alpha*beta'*Z1);
elseif (isempty(Z2)==0)&(r==0);
  epsilon = Z0-(Psi*Z2);
elseif (isempty(Z2)==1)&(r>0);
  epsilon = Z0-(alpha*beta'*Z1);
else;
  epsilon = Z0;
end;
if r<n;
   C = cmat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0);
else;
   C = cmat_stat(Psi,alpha,beta,epsilon,Z1,Z2,tm.p,tm.d,0,[]);
end;
disp(B);
disp(C*B);

%
% close down waitbar
%

if (inside_waitbar==0)&(waitbar==1);
   set(mywait,'UserData','done');
   close(mywait);
   drawnow;
end;













%
% end of fiml.m
%
